/* test0c.c - mouse (1) */

#include <mikeos.h>

/*
	used MikeOS API :-

	mikeos_mouse_disable
	mikeos_mouse_init
	mikeos_mouse_set_h_limit
	mikeos_mouse_set_text_cursor
	mikeos_mouse_set_position
	mikeos_mouse_set_user_subroutine
	mikeos_mouse_set_v_limit
	mikeos_mouse_show

	mikeos_clear_screen
	mikeos_move_cursor
	mikeos_print_4hex
	mikeos_print_newline
	mikeos_print_space
	mikeos_print_string
*/

static	unsigned int	Event_State = 0;
static	unsigned int	Button_State = 0;
static	int		X_Pos = 0;
static	int		Y_Pos = 0;
static	int		X_Delta_Pos = 0;
static	int		Y_Delta_Pos = 0;
static	int		Event_Ready = 0;

/* mouse event handler, ASM-C glue */
static	void	mouse_event_handler(void)
{
#asm
	pushad		! interrupt handler saves 16-bit register,
			! but libmikeos uses 32bit register for long ops.
			! of course if you write pure 16-bit code,
			! you can remove pushad/popad here.

	push	di	! Y delta position
	push	si	! X delta position
	push	dx	! Y position
	push	cx	! X position
	push	bx	! button state	
	push	ax	! event state

	! void mouse_event_main(unsigned int event_state,
	!			unsigned int button_state,
	!			int x_pos, int y_pos,
	!			int x_delta_pos, int y_delta_pos);
	call	_mouse_event_main
	add	sp, #12

	popad
	retf		! FAR return is required	
#endasm

	/* "return" adds garbage */
}

/* mouse event handler, C */
static	void	mouse_event_main(unsigned int event_state, unsigned int button_state, int x_pos, int y_pos, int x_delta_pos, int y_delta_pos)
{
	/* if main_loop is not ACKed, ignore event */
	if (!Event_Ready) {
		Event_State = event_state;
		Button_State = button_state;
		X_Pos = x_pos;
		Y_Pos = y_pos;
		X_Delta_Pos = x_delta_pos;
		Y_Delta_Pos = y_delta_pos;
		Event_Ready = 1;
	}

	return;
}

/* test mouse function call (1) */
static	void	mouse_test1(void)
{
	int	key;
	static	char	str0[] = "mouse_test1: press [ESC] to continue";

	mikeos_clear_screen();
	mikeos_print_string(str0);

	mikeos_mouse_init(NULL);
	mikeos_mouse_set_text_cursor(0xffff, 0xff00);
	mikeos_mouse_set_h_limit(2, 77);
	mikeos_mouse_set_v_limit(2, 22);
	mikeos_mouse_set_user_subroutine(0x1f, mouse_event_handler);
	mikeos_mouse_show();
	mikeos_mouse_set_position(4, 8);

	do {
		while (!Event_Ready);	/* wait for event */

		mikeos_move_cursor(0, 12);
		mikeos_print_4hex(Event_State);
		mikeos_print_space();
		mikeos_print_4hex(Button_State);
		mikeos_print_space();
		mikeos_print_4hex(X_Pos);
		mikeos_print_space();
		mikeos_print_4hex(Y_Pos);
		mikeos_print_space();
		mikeos_print_4hex(X_Delta_Pos);
		mikeos_print_space();
		mikeos_print_4hex(Y_Delta_Pos);
		mikeos_print_space();

		Event_Ready = 0;	/* receive next event */

	} while (mikeos_check_for_key() != 0x1b);

	mikeos_mouse_disable();
	mikeos_print_newline();

	return;
}


int	MikeMain(void *arg)
{
	mouse_test1();

	return 0;
}
